package com.broadlink.nj.flutter.plugin.blaccount_flutter_plugin;

import androidx.annotation.NonNull;

import java.util.concurrent.Callable;

import cn.com.broadlink.account.BLAccount;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers;
import io.reactivex.rxjava3.core.Observable;
import io.reactivex.rxjava3.core.Observer;
import io.reactivex.rxjava3.core.Scheduler;
import io.reactivex.rxjava3.disposables.Disposable;
import io.reactivex.rxjava3.schedulers.Schedulers;

/**
 * BlaccountFlutterPlugin
 */
public class BlaccountFlutterPlugin implements FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;

    private final String LOGTAG = BlaccountFlutterPlugin.class.getSimpleName();

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "blaccount_flutter_plugin");
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        BLLogUtils.d(LOGTAG, "onMethodCall:" + call.method);
        final Result finalResult = result;
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        }
        else if (call.method.equals(NativeInterface.INIT_SDK)) {
            boolean init_result = BLAccountTools.getInstance().init();
            result.success(init_result);
        }
        else if (call.method.equals(NativeInterface.LOGIN)) {
            final String account = call.argument("account");
            final String password = call.argument("password");
            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String login_result = BLAccountTools.getInstance().login(account, password);
                    return login_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String login_result) {
                            finalResult.success(login_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.LOCAL_LOGIN)) {
            final String loginSession = call.argument("login_session");
            final String userId = call.argument("user_id");
            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String login_result = BLAccountTools.getInstance().localLogin(loginSession, userId);
                    return login_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String login_result) {
                            finalResult.success(login_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.SEND_PHONE_REG_VCODE)) {
            final String phone = call.argument("phone");
            final String countryCode = call.argument("country_code");
            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String send_result = BLAccountTools.getInstance().sendRegVCode(phone, countryCode);
                    return send_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String send_result) {
                            finalResult.success(send_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.SEND_EMAIL_REG_VCODE)) {
            final String email = call.argument("email");

            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String send_result = BLAccountTools.getInstance().sendRegVCode(email);
                    return send_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String send_result) {
                            finalResult.success(send_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.REGIST)) {
            final String data = call.argument("data");

            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String regist_result = BLAccountTools.getInstance().regist(data);
                    return regist_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String regist_result) {
                            finalResult.success(regist_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.SEND_RESET_PWD_VCODE)) {
            final String account = call.argument("account");
            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String regist_result = BLAccountTools.getInstance().sendResetPwdVCode(account);
                    return regist_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String regist_result) {
                            finalResult.success(regist_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.RESET_PASSWORD)) {
            final String account = call.argument("account");
            final String vcode = call.argument("vcode");
            final String password = call.argument("password");

            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String reset_pwd_result = BLAccountTools.getInstance().resetPwd(account, vcode, password);
                    return reset_pwd_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String reset_pwd_result) {
                            finalResult.success(reset_pwd_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.SEND_MODIFY_EMAIL_VCODE)) {
            final String email = call.argument("email");
            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String send_result = BLAccountTools.getInstance().sendModifyEmailVCode(email);
                    return send_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String send_result) {
                            finalResult.success(send_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.SEND_MODIFY_PHONE_VCODE)) {
            final String phone = call.argument("phone");
            final String countryCode = call.argument("country_code");
            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String send_result = BLAccountTools.getInstance().sendModifyPhoneVCode(phone, countryCode);
                    return send_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String send_result) {
                            finalResult.success(send_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.MODIFY_PHONE)) {
            final String phone = call.argument("phone");
            final String countryCode = call.argument("country_code");
            final String vcode = call.argument("vcode");
            final String password = call.argument("password");

            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String modify_result = BLAccountTools.getInstance().modifyPhone(phone, countryCode, vcode, password);
                    return modify_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String modify_result) {
                            finalResult.success(modify_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.MODIFY_EMAIL)) {
            final String email = call.argument("email");
            final String vcode = call.argument("vcode");
            final String password = call.argument("password");

            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String modify_result = BLAccountTools.getInstance().modifyEmail(email, vcode, password);
                    return modify_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String modify_result) {
                            finalResult.success(modify_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.MODIFY_PASSWORD)) {
            final String oldPassword = call.argument("old_password");
            final String newPassword = call.argument("new_password");

            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String modify_result = BLAccountTools.getInstance().modifyPassword(oldPassword, newPassword);
                    return modify_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String modify_result) {
                            finalResult.success(modify_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.MODIFY_NICKNAME)) {
            final String nickname = call.argument("nickname");

            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String modify_result = BLAccountTools.getInstance().modifyNickname(nickname);
                    return modify_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String modify_result) {
                            finalResult.success(modify_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.MODIFY_ICON)) {
            final String filePath = call.argument("file_path");

            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String modify_result = BLAccountTools.getInstance().modifyIcon(filePath);
                    return modify_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String modify_result) {
                            finalResult.success(modify_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        }
        else if (call.method.equals(NativeInterface.QUERY_USERS_INFO)) {
            final String data = call.argument("data");

            Observable.fromCallable(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    String users_info_result = BLAccountTools.getInstance().queryUsersInfo(data);
                    return users_info_result;
                }
            })
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Observer<String>() {
                        @Override
                        public void onSubscribe(@io.reactivex.rxjava3.annotations.NonNull Disposable d) {

                        }

                        @Override
                        public void onNext(@io.reactivex.rxjava3.annotations.NonNull String users_info_result) {
                            finalResult.success(users_info_result);
                        }

                        @Override
                        public void onError(@io.reactivex.rxjava3.annotations.NonNull Throwable e) {
                            finalResult.success(null);
                        }

                        @Override
                        public void onComplete() {

                        }
                    });
        } else {
            result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }
}
