////
////  AppSDKManager.m
////  blaccount_flutter_plugin
////
////  Created by 王振 on 2021/8/24.
////
//
#import "AppSDKManager.h"
#import <BLLetCore/BLLetCore.h>
#import <BLLetBase/BLLetBase.h>
#import <BLLetFamily/BLLetFamily.h>
#import <BLLetAccount/BLLetAccount.h>
#import <JSONKit-BroadLink/JSONKit.h>

@interface AppSDKManager ()

@property (strong,nonatomic)NSMutableArray <BLDNADevice *>*localDeviceArray;

@end

@implementation AppSDKManager

+(instancetype)sharedManager {
    static dispatch_once_t onceToken;
    static AppSDKManager *manager = nil;
    dispatch_once(&onceToken, ^{
        manager = [[self alloc] init];
    });
    return manager;
}
-(NSMutableArray<BLDNADevice *> *)localDeviceArray {
    if (_localDeviceArray == nil) {
        _localDeviceArray = [NSMutableArray array];
    }
    return _localDeviceArray;
}
-(BOOL)blAppSDKAccountInit {
    [BLAccount sharedAccount];
    return YES;
}
-(void)loginWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] login:[info objectForKey:@"account"] password:[info objectForKey:@"password"] completionHandler:^(BLLoginResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"msg":result.msg,
                @"data":@{//nickname,icon_path,sex,country_code,phone,email,birthday,user_type
                        @"login_session":result.loginsession?result.loginsession:@"",
                        @"user_id":result.userid?result.userid:@"",
                        @"nickname":result.nickname?result.nickname:@"",
                        @"icon_path":result.iconpath?result.iconpath:@"",
                        @"sex":result.sex?result.sex:@"",
                        @"country_code":result.countrycode?result.countrycode:@"",
                        @"phone":result.phone?result.phone:@"",
                        @"email":result.email?result.email:@"",
                        @"birthday":result.birthday?result.birthday:@"",
                        @"user_type":result.usertype?result.usertype:@"",
                }
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}

-(void)localLoginWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] localLoginWithUsrid:[info objectForKey:@"user_id"] session:[info objectForKey:@"login_session"] completionHandler:^(BLLoginResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"mas":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
-(void)sendPhoneCodeForRegisterWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] sendRegVCode:[info objectForKey:@"phone"] countryCode:[info objectForKey:@"country_code"] completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"mas":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}

-(void)sendEmailCodeForRegisterWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] sendRegVCode:[info objectForKey:@"email"] completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"mas":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
-(void)registerAccountWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] regist:[info objectForKey:@"account"] password:[info objectForKey:@"password"] nickname:[info objectForKey:@"nickname"] vcode:[info objectForKey:@"code"] sex:BL_ACCOUNT_MALE birthday:[info objectForKey:@"birthday"] countryCode:[info objectForKey:@"country_code"] iconPath:[info objectForKey:@"iconpath"] completionHandler:^(BLLoginResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"msg":result.msg,
                @"data":@{//nickname,icon_path,sex,country_code,phone,email,birthday,user_type
                        @"login_session":result.loginsession?result.loginsession:@"",
                        @"user_id":result.userid?result.userid:@"",
                        @"nickname":result.nickname?result.nickname:@"",
                        @"icon_path":result.iconpath?result.iconpath:@"",
                        @"sex":result.sex?result.sex:@"",
                        @"country_code":result.countrycode?result.countrycode:@"",
                        @"phone":result.phone?result.phone:@"",
                        @"email":result.email?result.email:@"",
                        @"birthday":result.birthday?result.birthday:@"",
                        @"user_type":result.usertype?result.usertype:@"",
                }
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//重置密码发送验证码
-(void)resetPwdSendVCodeWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] sendModifyVCode:[info objectForKey:@"account"] completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"mas":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//重置密码
-(void)resetPwdWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] retrivePassword:[info objectForKey:@"account"] vcode:[info objectForKey:@"vcode"] newPassword:[info objectForKey:@"password"] completionHandler:^(BLLoginResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"msg":result.msg,
                @"data":@{//nickname,icon_path,sex,country_code,phone,email,birthday,user_type
                        @"login_session":result.loginsession?result.loginsession:@"",
                        @"user_id":result.userid?result.userid:@"",
                        @"nickname":result.nickname?result.nickname:@"",
                        @"icon_path":result.iconpath?result.iconpath:@"",
                        @"sex":result.sex?result.sex:@"",
                        @"country_code":result.countrycode?result.countrycode:@"",
                        @"phone":result.phone?result.phone:@"",
                        @"email":result.email?result.email:@"",
                        @"birthday":result.birthday?result.birthday:@"",
                        @"user_type":result.usertype?result.usertype:@"",
                }
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//发送修改手机号的验证码，
-(void)sendVCodeToModifyPhoneWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] sendModifyVCode:[info objectForKey:@"phone"] countryCode:[info objectForKey:@"country_code"] completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"mas":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//发送修改邮箱的验证码
-(void)sendVCodeToModifyEmailWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] sendModifyVCode:[info objectForKey:@"email"] completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"mas":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//修改手机号
-(void)modifyPhoneWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] modifyPhone:[info objectForKey:@"phone"] countryCode:[info objectForKey:@"country_code"] vcode:[info objectForKey:@"vcode"] password:[info objectForKey:@"password"] newpassword:nil completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"mas":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//修改邮箱
-(void)modifyEmailWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] modifyEmail:[info objectForKey:@"email"] vcode:[info objectForKey:@"vcode"] password:[info objectForKey:@"password"] newpassword:nil completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"mas":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//修改密码
-(void)modifyPwdWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] modifyPassword:[info objectForKey:@"old_password"] newPassword:[info objectForKey:@"new_password"] completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"mas":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//修改昵称
-(void)modifyNickNameWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] modifyUserNickname:[info objectForKey:@"nicknam"] completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"mas":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//修改头像
-(void)modifyIncopathWithInfo:(NSDictionary *)info uccess:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] modifyUserIcon:[info objectForKey:@"file_path"] completionHandler:^(BLModifyUserIconResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(1),
                @"mas":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
-(void)getUserInfoWithInfo:(NSDictionary *)info uccess:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    NSArray *userIdArr = [self toArrayOrNSDictionary:[info objectForKey:@"data"]];
    [[BLAccount sharedAccount] getUserInfo:userIdArr completionHandler:^(BLGetUserInfoResult * _Nonnull result) {
        if ([result succeed]) {
            NSMutableArray *array = [NSMutableArray array];
            for (BLUserInfo *userInfo in result.info) {
                NSDictionary *dic = @{
                    @"user_id":userInfo.userid?userInfo.userid:@"",
                    @"nickname":userInfo.nickname?userInfo.nickname:@"",
                    @"icon":userInfo.iconUrl?userInfo.iconUrl:@""
                };
                [array addObject:dic];
            }
            NSDictionary *resultDic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@"",
                @"data":array
            };
            success(resultDic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"mas":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}









- (id)toArrayOrNSDictionary:(NSString *)jsonData{

    if (jsonData != nil) {
        NSData* data = [jsonData dataUsingEncoding:NSUTF8StringEncoding];
        id jsonObject = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingAllowFragments error:nil];
        if (jsonObject != nil){
            return jsonObject;
        }else{
            // 解析错误
            return nil;

        }
    }
    return nil;
}



- (NSString *)p_toJsonString:(NSDictionary *)dic {
    if (dic == nil) {
        return @"";
    }
    NSError *error;
    NSData *data = [NSJSONSerialization dataWithJSONObject:dic options:0 error:&error];
    if (error) {
        NSLog(@"%@", error.localizedDescription);
        return nil;
    }

    return [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
}

- (NSDictionary *)p_toDictory:(id)json {
    if (!json || json == (id)kCFNull) return nil;
    NSDictionary *dic = nil;
    NSData *jsonData = nil;
    if ([json isKindOfClass:[NSDictionary class]]) {
        dic = json;
    } else if ([json isKindOfClass:[NSString class]]) {
        jsonData = [(NSString *)json dataUsingEncoding : NSUTF8StringEncoding];
    } else if ([json isKindOfClass:[NSData class]]) {
        jsonData = json;
    }
    if (jsonData) {
        dic = [NSJSONSerialization JSONObjectWithData:jsonData options:kNilOptions error:NULL];
        if (![dic isKindOfClass:[NSDictionary class]]) dic = nil;
    }
    return dic;
}
#pragma mark - BLControllerDelegate
- (Boolean)filterDevice:(BLDNADevice *)device {
    return NO;
}

- (Boolean)shouldAdd:(BLDNADevice *)device {
    //本地搜索到设备不会自动添加到SDK内
    return NO;
}

- (void)onDeviceUpdate:(BLDNADevice *)device isNewDevice:(Boolean)isNewDevice {
    BOOL isContain = NO;
    for (BLDNADevice *tempDevice in self.localDeviceArray) {
        if ([tempDevice.did isEqualToString:device.did]) {
            isContain = YES;
            tempDevice.controlId = device.controlId;
            tempDevice.controlKey = device.controlKey;
            tempDevice.lanaddr = device.lanaddr;
            tempDevice.newConfig = device.newConfig;
            break;
        }
    }
    if (!isContain) {
        [self.localDeviceArray addObject:device];
    }
}

- (NSString *)sharedPath {
    BLConfigParam *config = [BLConfigParam sharedConfigParam];
    return config.sdkFileSavePath;
}


@end
